<?php

/*
 * Returns a resource id for a string value.
 * Ignores string values that are listed in the array $resource_value_blacklist.
 */
function get_resource_id($default_value) {
	global $database_new, $resource_value_blacklist;
	//	$default_value = trim(str_replace(':', '', $default_value));
	$resource_id = null;
	if (in_array($default_value, $resource_value_blacklist)) {
		echo "INFO: Ignoring request for blacklisted resource id: '$default_value'<br>";
	} else {
		$results = $database_new->query("SELECT name FROM strings WHERE LOWER(defaultValue)=LOWER('$default_value') LIMIT 1;");
		if ($row = $results->fetchArray()) {
			$resource_id = $row['name'];
		} else {
			echo "INFO: Resource not found in database: '$default_value'<br>";
			if ($database_new->query("INSERT INTO strings(defaultValue) VALUES('$default_value')") === false) {
				echo "ERROR: Failed to insert new resource: '$default_value'";
				echo $database_new->lastErrorMsg();
				echo "<br>";
			} else {
				echo "INFO: Inserted new resource '$default_value'<br>";
				$last_insert_row_id = $database_new->lastInsertRowID();
				$resource_id = "string_$last_insert_row_id";
			}
		}
		//	echo "defaultValue '$default_value', resourceId '$resource_id'<br>";
	}
	return $resource_id;
}

/*
 * Localizes the b4a View represented by the JSON object named $layout_object.
 * Ignores Views that are listed in the array $view_name_blacklist.
 */
function localize_kids_object($kids_object, $layout_object) {
	global $view_name_blacklist;
	foreach ($kids_object as $kid_id => &$kid) {
		//	my_var_dump($kid);
		//	echo $kid->{'javaType'};
		if(in_array($kid->{'name'}, $view_name_blacklist)){
			echo "Ignoring blacklisted View: '{$kid->{'name'}}'<br>";
		} else {
			switch ($kid->{'javaType'}) {
				case '.ButtonWrapper':
				case '.CompoundButtonWrapper$CheckBoxWrapper':
				case '.LabelWrapper':
				case '.CompoundButtonWrapper$RadioButtonWrapper':
					$text = trim($kid->{'text'});
					if (!is_null($text) && !$text == '') {
						$resource_id = get_resource_id($text);
						if (is_null($resource_id)) {
							echo "Ignoring View(null resource id): '{$kid->{'name'}}'<br>";
						} else {
							echo "Updating View: '{$kid->{'name'}}'<br>";
							$kid->{'tag'} = $resource_id;
							$kid->{'text'} = '';
						}
					}
					break;
				case '.EditTextWrapper':
					$hint = trim($kid->{'hint'});
					if (!is_null($hint) && !$hint == '') {
						$resource_id = get_resource_id($hint);
						if (is_null($resource_id)) {
							echo "Ignoring View(null resource id): '{$kid->{'name'}}'<br>";
						} else {
							echo "Updating View: '{$kid->{'name'}}'<br>";
							$kid->{'tag'} = $resource_id;
							$kid->{'hint'} = '';
						}
					}
					break;
				case '.PanelWrapper':
					if (isset($kid->{':kids'})) {
						localize_kids_object($kid->{':kids'}, $layout_object);
					}
					break;
				case '.SpinnerWrapper':
					if (in_array($kid->{'name'}, $view_name_blacklist)) {
						echo "Ignoring blacklisted Spinner: '{$kid->{'name'}}'<br>";
					} else {
						echo "Updating Spinner: '{$kid->{'name'}}'<br>";
						$kids_object->{$kid_id} = localize_spinner($kid, $layout_object);
					}
					break;
			}
		}
	}
}

/*
 * Replaces a standard b4a Spinner with a custom (DesignerView) Spinner.
 * All b4a Spinner properties are copied to the new custom Spinner.
 */
function localize_spinner($spinner_object, $layout_object) {
	//	need a default LocalizedSpinner object
	$localized_spinner_object = json_decode(file_get_contents(LOCALIZEDSPINNER_TEMPLATE_PATH), false);

	//	my_var_dump($localized_spinner_object);
	//	clone all Spinner properties to the LocalizedSpinner
	foreach (array('parent', 'visible', 'fontsize', 'textColor', 'enabled', 'name', 'eventName', 'prompt', 'variant0', 'variant1', 'variant2') as $property) {
		$localized_spinner_object->{$property} = $spinner_object->{$property};
	}

	//	update LayoutHeader.ControlsHeaders
	foreach ($layout_object->{'LayoutHeader'}->{'ControlsHeaders'} as &$controls_headers) {
		if ($controls_headers->{'Name'} == $spinner_object->{'name'}) {
			$controls_headers->{'DesignerType'} = 'CustomView';
			$controls_headers->{'JavaType'} = '.CustomViewWrapper';
			break;
		}
	}

	return $localized_spinner_object;
}

function my_var_dump($value) {
	echo '<hr><pre>' . print_r($value, true) . '</pre><hr>';
	//	echo '<pre>'.json_encode($value).'</pre>';
}

define('DATABASE_PATH', 'path_to_my_database.sqlite');
define('DEST_PATH', 'json_updated');
//	LocalizedSpinner.json is a JSON representation of a custom (DesignerView) Spinner
define('LOCALIZEDSPINNER_TEMPLATE_PATH', 'templates/LocalizedSpinner.json');
define('SOURCE_PATH', 'json_original');

$database_new = new SQLite3(DATABASE_PATH);
$directory_iterator = new DirectoryIterator(SOURCE_PATH);
$layout_file_blacklist = array();
$resource_value_blacklist=array('', '↓', '↑');
$view_name_blacklist = array('edtiFIMSVPin', 'edtiPassword', 'spnFactory', 'spnFromWell', 'spnHold', 'spnRejWell', 'spnSchedule2ver', 'spnToWell', 'spnWell', 'spnYear');	//	TODO typos to fix in original project source?

foreach ($directory_iterator as $xml_file) {

	if ($xml_file->isFile()) {
		$json_file_filename = $xml_file->getFilename();

		if (in_array($json_file_filename, $layout_file_blacklist)) {
			echo "Ignoring blacklisted layout file: '$json_file_filename'<br>";
		} else {
			$json_string = file_get_contents(SOURCE_PATH . "/$json_file_filename");
			if ($json_string === false) {
				echo "Failed to open layout file: '$json_file_filename'<br>";
			} else {
				$layout_object = json_decode($json_string, false);
				if ($layout_object === null) {
					echo "Failed to decode json in layout file: '$json_file_filename'";
				} else {
					echo "Updating layout file: '<b>$json_file_filename</b>'<br>";
					localize_kids_object($layout_object->{'Data'}->{':kids'}, $layout_object);
					echo '<hr>';

					file_put_contents(DEST_PATH . "/$json_file_filename", json_encode($layout_object));
				}
			}
		}
	}
}

$database_new->close();
?>
